@@ -4,26 +4,23 @@
 
 use std::collections::HashMap;
 use std::default::Default;
-use std::fmt;
-use std::fs::File;
-use std::io::prelude::*;
-use std::io::{self, BufReader};
-use std::path::Path as StdPath;
 use std::str::FromStr;
+use std::{fmt, fs, path::Path as StdPath};
 
 use serde::de::value::{MapAccessDeserializer, SeqAccessDeserializer};
 use serde::de::{Deserialize, Deserializer, MapAccess, SeqAccess, Visitor};
 
 use toml;
 
-use bindgen::ir::annotation::AnnotationSet;
-use bindgen::ir::path::Path;
-pub use bindgen::rename::RenameRule;
+use crate::bindgen::ir::annotation::AnnotationSet;
+use crate::bindgen::ir::path::Path;
+use crate::bindgen::ir::repr::ReprAlign;
+pub use crate::bindgen::rename::RenameRule;
 
-pub const VERSION: &'static str = env!("CARGO_PKG_VERSION");
+pub const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 /// A language type to generate bindings for.
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum Language {
     Cxx,
     C,
@@ -100,8 +97,36 @@ impl FromStr for Layout {
 
 deserialize_enum_str!(Layout);
 
+/// How the comments containing documentation should be styled.
+#[derive(Debug, Clone, PartialEq, Copy)]
+pub enum DocumentationStyle {
+    C,
+    C99,
+    Doxy,
+    Cxx,
+    Auto,
+}
+
+impl FromStr for DocumentationStyle {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<DocumentationStyle, Self::Err> {
+        match s.to_lowercase().as_ref() {
+            "c" => Ok(DocumentationStyle::C),
+            "c99" => Ok(DocumentationStyle::C99),
+            "cxx" => Ok(DocumentationStyle::Cxx),
+            "c++" => Ok(DocumentationStyle::Cxx),
+            "doxy" => Ok(DocumentationStyle::Doxy),
+            "auto" => Ok(DocumentationStyle::Auto),
+            _ => Err(format!("Unrecognized documentation style: '{}'.", s)),
+        }
+    }
+}
+
+deserialize_enum_str!(DocumentationStyle);
+
 /// A style of Style to use when generating structs and enums.
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum Style {
     Both,
     Tag,
@@ -109,19 +134,17 @@ pub enum Style {
 }
 
 impl Style {
-    pub fn generate_tag(&self) -> bool {
+    pub fn generate_tag(self) -> bool {
         match self {
-            &Style::Both => true,
-            &Style::Tag => true,
-            &Style::Type => false,
+            Style::Both | Style::Tag => true,
+            Style::Type => false,
         }
     }
 
-    pub fn generate_typedef(&self) -> bool {
+    pub fn generate_typedef(self) -> bool {
         match self {
-            &Style::Both => true,
-            &Style::Tag => false,
-            &Style::Type => true,
+            Style::Both | Style::Type => true,
+            Style::Tag => false,
         }
     }
 }
@@ -197,6 +220,8 @@ pub struct ExportConfig {
     pub prefix: Option<String>,
     /// Types of items to generate.
     pub item_types: Vec<ItemType>,
+    /// Whether renaming overrides or extends prefixing.
+    pub renaming_overrides_prefixing: bool,
 }
 
 impl ExportConfig {
@@ -211,6 +236,9 @@ impl ExportConfig {
     pub(crate) fn rename(&self, item_name: &mut String) {
         if let Some(name) = self.rename.get(item_name) {
             *item_name = name.clone();
+            if self.renaming_overrides_prefixing {
+                return;
+            }
         }
         if let Some(ref prefix) = self.prefix {
             item_name.insert_str(0, &prefix);
@@ -218,6 +246,29 @@ impl ExportConfig {
     }
 }
 
+/// Settings to apply to generated types with layout modifiers.
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(rename_all = "snake_case")]
+#[serde(deny_unknown_fields)]
+#[serde(default)]
+pub struct LayoutConfig {
+    /// The way to annotate C types as #[repr(packed)].
+    pub packed: Option<String>,
+    /// The way to annotate C types as #[repr(align(...))]. This is assumed to be a functional
+    /// macro which takes a single argument (the alignment).
+    pub aligned_n: Option<String>,
+}
+
+impl LayoutConfig {
+    pub(crate) fn ensure_safe_to_represent(&self, align: &ReprAlign) -> Result<(), String> {
+        match (align, &self.packed, &self.aligned_n) {
+            (ReprAlign::Packed, None, _) => Err("Cannot safely represent #[repr(packed)] type without configured 'packed' annotation.".to_string()),
+            (ReprAlign::Align(_), _, None) => Err("Cannot safely represent #[repr(aligned(...))] type without configured 'aligned_n' annotation.".to_string()),
+            _ => Ok(()),
+        }
+    }
+}
+
 /// Settings to apply to generated functions.
 #[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
@@ -228,6 +279,8 @@ pub struct FunctionConfig {
     pub prefix: Option<String>,
     /// Optional text to output after each function declaration
     pub postfix: Option<String>,
+    /// The way to annotation this function as #[must_use].
+    pub must_use: Option<String>,
     /// The style to layout the args
     pub args: Layout,
     /// The rename rule to apply to function args
@@ -239,6 +292,7 @@ impl Default for FunctionConfig {
         FunctionConfig {
             prefix: None,
             postfix: None,
+            must_use: None,
             args: Layout::Auto,
             rename_args: None,
         }
@@ -262,7 +316,7 @@ impl FunctionConfig {
 }
 
 /// Settings to apply to generated structs.
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Default, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
 #[serde(deny_unknown_fields)]
 #[serde(default)]
@@ -284,21 +338,11 @@ pub struct StructConfig {
     pub derive_gt: bool,
     /// Whether to generate a greater than or equal to operator on structs with one field
     pub derive_gte: bool,
-}
-
-impl Default for StructConfig {
-    fn default() -> StructConfig {
-        StructConfig {
-            rename_fields: None,
-            derive_constructor: false,
-            derive_eq: false,
-            derive_neq: false,
-            derive_lt: false,
-            derive_lte: false,
-            derive_gt: false,
-            derive_gte: false,
-        }
-    }
+    /// Whether associated constants should be in the body. Only applicable to
+    /// non-transparent structs, and in C++-only.
+    pub associated_constants_in_body: bool,
+    /// The way to annotate this struct as #[must_use].
+    pub must_use: Option<String>,
 }
 
 impl StructConfig {
@@ -362,6 +406,31 @@ pub struct EnumConfig {
     /// Whether to generate static `::X(..)` constructors and `IsX()`
     /// methods for tagged enums.
     pub derive_helper_methods: bool,
+    /// Whether to generate `AsX() const` methods for tagged enums.
+    pub derive_const_casts: bool,
+    /// Whether to generate `AsX()` methods for tagged enums.
+    pub derive_mut_casts: bool,
+    /// The name of the macro to use for `derive_{const,mut}casts`. If custom, you're
+    /// responsible to provide the necessary header, otherwise `assert` will be
+    /// used, and `<cassert>` will be included.
+    pub cast_assert_name: Option<String>,
+    /// The way to annotation this enum as #[must_use].
+    pub must_use: Option<String>,
+    /// Whether to generate destructors of tagged enums.
+    pub derive_tagged_enum_destructor: bool,
+    /// Whether to generate copy-constructors of tagged enums.
+    pub derive_tagged_enum_copy_constructor: bool,
+    /// Whether to generate copy-assignment operators of tagged enums.
+    ///
+    /// This is only generated if a copy constructor for the same tagged enum is
+    /// generated as well.
+    pub derive_tagged_enum_copy_assignment: bool,
+    /// Declare the enum as an enum class.
+    /// Only relevant when targeting C++.
+    pub enum_class: bool,
+    /// Whether to generate empty, private default-constructors for tagged
+    /// enums.
+    pub private_default_tagged_enum_constructor: bool,
 }
 
 impl Default for EnumConfig {
@@ -371,6 +440,15 @@ impl Default for EnumConfig {
             add_sentinel: false,
             prefix_with_name: false,
             derive_helper_methods: false,
+            derive_const_casts: false,
+            derive_mut_casts: false,
+            cast_assert_name: None,
+            must_use: None,
+            derive_tagged_enum_destructor: false,
+            derive_tagged_enum_copy_constructor: false,
+            derive_tagged_enum_copy_assignment: false,
+            enum_class: true,
+            private_default_tagged_enum_constructor: false,
         }
     }
 }
@@ -388,6 +466,51 @@ impl EnumConfig {
         }
         self.derive_helper_methods
     }
+    pub(crate) fn derive_const_casts(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("derive-const-casts") {
+            return x;
+        }
+        self.derive_const_casts
+    }
+    pub(crate) fn derive_mut_casts(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("derive-mut-casts") {
+            return x;
+        }
+        self.derive_mut_casts
+    }
+    pub(crate) fn derive_tagged_enum_destructor(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("derive-tagged-enum-destructor") {
+            return x;
+        }
+        self.derive_tagged_enum_destructor
+    }
+    pub(crate) fn derive_tagged_enum_copy_constructor(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("derive-tagged-enum-copy-constructor") {
+            return x;
+        }
+        self.derive_tagged_enum_copy_constructor
+    }
+    pub(crate) fn derive_tagged_enum_copy_assignment(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("derive-tagged-enum-copy-assignment") {
+            return x;
+        }
+        self.derive_tagged_enum_copy_assignment
+    }
+    pub(crate) fn enum_class(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("enum-class") {
+            return x;
+        }
+        self.enum_class
+    }
+    pub(crate) fn private_default_tagged_enum_constructor(
+        &self,
+        annotations: &AnnotationSet,
+    ) -> bool {
+        if let Some(x) = annotations.bool("private-default-tagged-enum-constructor") {
+            return x;
+        }
+        self.private_default_tagged_enum_constructor
+    }
 }
 
 /// Settings to apply to generated constants.
@@ -408,6 +531,16 @@ impl Default for ConstantConfig {
     }
 }
 
+/// Settings for custom macro expansion.
+#[derive(Debug, Clone, Deserialize, Default)]
+#[serde(rename_all = "snake_case")]
+#[serde(deny_unknown_fields)]
+#[serde(default)]
+pub struct MacroExpansionConfig {
+    /// Whether the `bitflags` macro should be expanded.
+    pub bitflags: bool,
+}
+
 /// Settings to apply when running `rustc --pretty=expanded`
 #[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
@@ -475,7 +608,7 @@ fn retrocomp_parse_expand_config_deserialize<'de, D: Deserializer<'de>>(
 }
 
 /// Settings to apply when parsing.
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Default, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
 #[serde(deny_unknown_fields)]
 #[serde(default)]
@@ -499,17 +632,23 @@ pub struct ParseConfig {
     /// Whether to use a new temporary target directory when running `rustc --pretty=expanded`.
     /// This may be required for some build processes.
     pub clean: bool,
+    /// List of crate names which generate consts, statics, and fns. By default
+    /// no dependent crates generate them.
+    pub extra_bindings: Vec<String>,
 }
 
-impl Default for ParseConfig {
-    fn default() -> ParseConfig {
-        ParseConfig {
-            parse_deps: false,
-            include: None,
-            exclude: Vec::new(),
-            expand: ParseExpandConfig::default(),
-            clean: false,
+impl ParseConfig {
+    pub(crate) fn should_generate_top_level_item(
+        &self,
+        crate_name: &str,
+        binding_crate_name: &str,
+    ) -> bool {
+        if crate_name == binding_crate_name {
+            // Always generate items for the binding crate.
+            return true;
         }
+
+        self.extra_bindings.iter().any(|dep| dep == crate_name)
     }
 }
 
@@ -542,6 +681,8 @@ pub struct Config {
     pub namespace: Option<String>,
     /// An optional list of namespaces. Only applicable when language="C++"
     pub namespaces: Option<Vec<String>>,
+    /// An optional list of namespaces to declare as using. Only applicable when language="C++"
+    pub using_namespaces: Option<Vec<String>>,
     /// The style to use for braces
     pub braces: Braces,
     /// The preferred length of a line, used for auto breaking function arguments
@@ -550,12 +691,18 @@ pub struct Config {
     pub tab_width: usize,
     /// The language to output bindings for
     pub language: Language,
+    /// Include preprocessor defines in C bindings to ensure C++ compatibility
+    pub cpp_compat: bool,
     /// The style to declare structs, enums and unions in for C
     pub style: Style,
     /// The configuration options for parsing
     pub parse: ParseConfig,
     /// The configuration options for exporting
     pub export: ExportConfig,
+    /// The configuration options for macros.
+    pub macro_expansion: MacroExpansionConfig,
+    /// The configuration options for type layouts.
+    pub layout: LayoutConfig,
     /// The configuration options for functions
     #[serde(rename = "fn")]
     pub function: FunctionConfig,
@@ -572,6 +719,8 @@ pub struct Config {
     pub defines: HashMap<String, String>,
     /// Include doc comments from rust as documentation
     pub documentation: bool,
+    /// How documentation comments should be styled.
+    pub documentation_style: DocumentationStyle,
 }
 
 impl Default for Config {
@@ -587,34 +736,36 @@ impl Default for Config {
             no_includes: false,
             namespace: None,
             namespaces: None,
+            using_namespaces: None,
             braces: Braces::SameLine,
             line_length: 100,
             tab_width: 2,
             language: Language::Cxx,
+            cpp_compat: false,
             style: Style::Type,
+            macro_expansion: Default::default(),
             parse: ParseConfig::default(),
             export: ExportConfig::default(),
+            layout: LayoutConfig::default(),
             function: FunctionConfig::default(),
             structure: StructConfig::default(),
             enumeration: EnumConfig::default(),
             constant: ConstantConfig::default(),
             defines: HashMap::new(),
             documentation: true,
+            documentation_style: DocumentationStyle::Auto,
         }
     }
 }
 
 impl Config {
     pub fn from_file<P: AsRef<StdPath>>(file_name: P) -> Result<Config, String> {
-        fn read(file_name: &StdPath) -> io::Result<String> {
-            let file = File::open(file_name)?;
-            let mut reader = BufReader::new(&file);
-            let mut contents = String::new();
-            reader.read_to_string(&mut contents)?;
-            Ok(contents)
-        }
-
-        let config_text = read(file_name.as_ref()).unwrap();
+        let config_text = fs::read_to_string(file_name.as_ref()).or_else(|_| {
+            Err(format!(
+                "Couldn't open config file: {}.",
+                file_name.as_ref().display()
+            ))
+        })?;
 
         match toml::from_str::<Config>(&config_text) {
             Ok(x) => Ok(x),
