@@ -6,19 +6,19 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::{Config, Language};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::ir::{
+use crate::bindgen::config::{Config, Language};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, GenericPath, Item,
     ItemContainer, Path, Repr, ReprStyle, ReprType, Struct, ToCondition, Type,
 };
-use bindgen::library::Library;
-use bindgen::mangle;
-use bindgen::monomorph::Monomorphs;
-use bindgen::rename::{IdentifierType, RenameRule};
-use bindgen::utilities::find_first_some;
-use bindgen::writer::{ListType, Source, SourceWriter};
+use crate::bindgen::library::Library;
+use crate::bindgen::mangle;
+use crate::bindgen::monomorph::Monomorphs;
+use crate::bindgen::rename::{IdentifierType, RenameRule};
+use crate::bindgen::utilities::find_first_some;
+use crate::bindgen::writer::{ListType, Source, SourceWriter};
 
 #[derive(Debug, Clone)]
 pub struct EnumVariant {
@@ -32,9 +32,7 @@ pub struct EnumVariant {
 fn value_from_expr(val: &syn::Expr) -> Option<i64> {
     match *val {
         syn::Expr::Lit(ref lit) => match lit.lit {
-            syn::Lit::Int(ref lit) if lit.value() <= ::std::i64::MAX as u64 => {
-                Some(lit.value() as i64)
-            }
+            syn::Lit::Int(ref lit) => lit.base10_parse::<i64>().ok(),
             _ => None,
         },
         syn::Expr::Unary(ref unary) => {
@@ -104,6 +102,7 @@ impl EnumVariant {
                     parse_fields(is_tagged, &fields.named)?,
                     is_tagged,
                     true,
+                    None,
                     false,
                     false,
                     Cfg::append(mod_cfg, Cfg::load(&variant.attrs)),
@@ -119,6 +118,7 @@ impl EnumVariant {
                     parse_fields(is_tagged, &fields.unnamed)?,
                     is_tagged,
                     true,
+                    None,
                     false,
                     true,
                     Cfg::append(mod_cfg, Cfg::load(&variant.attrs)),
@@ -252,8 +252,12 @@ impl Enum {
 
     pub fn load(item: &syn::ItemEnum, mod_cfg: Option<&Cfg>) -> Result<Enum, String> {
         let repr = Repr::load(&item.attrs)?;
-        if repr == Repr::RUST {
-            return Err("Enum not marked with a valid repr(prim) or repr(C).".to_owned());
+        if repr.style == ReprStyle::Rust && repr.ty.is_none() {
+            return Err("Enum is not marked with a valid #[repr(prim)] or #[repr(C)].".to_owned());
+        }
+        // TODO: Implement translation of aligned enums.
+        if repr.align.is_some() {
+            return Err("Enum is marked with #[repr(align(...))] or #[repr(packed)].".to_owned());
         }
 
         let generic_params = GenericParams::new(&item.generics);
@@ -414,7 +418,7 @@ impl Item for Enum {
                             &variant.export_name,
                             IdentifierType::EnumVariant(self),
                         ),
-                        variant.discriminant.clone(),
+                        variant.discriminant,
                         variant.body.as_ref().map(|body| {
                             (
                                 r.apply_to_snake_case(&body.0, IdentifierType::StructMember),
@@ -463,7 +467,7 @@ impl Item for Enum {
         let monomorph = Enum::new(
             mangled_path,
             GenericParams::default(),
-            self.repr.clone(),
+            self.repr,
             self.variants
                 .iter()
                 .map(|v| v.specialize(generic_values, &mappings))
@@ -490,10 +494,12 @@ impl Source for Enum {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
         let size = self.repr.ty.map(|ty| match ty {
             ReprType::USize => "uintptr_t",
+            ReprType::U64 => "uint64_t",
             ReprType::U32 => "uint32_t",
             ReprType::U16 => "uint16_t",
             ReprType::U8 => "uint8_t",
             ReprType::ISize => "intptr_t",
+            ReprType::I64 => "int64_t",
             ReprType::I32 => "int32_t",
             ReprType::I16 => "int16_t",
             ReprType::I8 => "int8_t",
@@ -537,11 +543,27 @@ impl Source for Enum {
 
             out.write("enum");
 
-            if !size.is_none() || config.style.generate_tag() {
+            if size.is_some() || config.style.generate_tag() {
                 write!(out, " {}", enum_name);
             }
+
+            if config.cpp_compat {
+                if let Some(prim) = size {
+                    out.new_line();
+                    out.write("#ifdef __cplusplus");
+                    out.new_line();
+                    write!(out, "  : {}", prim);
+                    out.new_line();
+                    out.write("#endif // __cplusplus");
+                    out.new_line();
+                }
+            }
         } else {
-            out.write("enum class");
+            if config.enumeration.enum_class(&self.annotations) {
+                out.write("enum class");
+            } else {
+                out.write("enum");
+            }
 
             if self.annotations.must_use {
                 if let Some(ref anno) = config.enumeration.must_use {
@@ -576,8 +598,18 @@ impl Source for Enum {
 
         if config.language == Language::C {
             if let Some(prim) = size {
+                if config.cpp_compat {
+                    out.new_line_if_not_start();
+                    out.write("#ifndef __cplusplus");
+                }
+
                 out.new_line();
                 write!(out, "typedef {} {};", prim, enum_name);
+
+                if config.cpp_compat {
+                    out.new_line_if_not_start();
+                    out.write("#endif // __cplusplus");
+                }
             }
         }
         // Done emitting the enum
@@ -782,12 +814,10 @@ impl Source for Enum {
                                 Type::MutRef(Box::new(return_type))
                             };
                             return_type.write(config, out);
+                        } else if const_casts {
+                            write!(out, "const {}&", body.export_name());
                         } else {
-                            if const_casts {
-                                write!(out, "const {}&", body.export_name());
-                            } else {
-                                write!(out, "{}&", body.export_name());
-                            }
+                            write!(out, "{}&", body.export_name());
                         }
 
                         write!(out, " As{}()", variant.export_name);
@@ -872,6 +902,23 @@ impl Source for Enum {
                 }
             }
 
+            if config.language == Language::Cxx
+                && config
+                    .enumeration
+                    .private_default_tagged_enum_constructor(&self.annotations)
+            {
+                out.new_line();
+                out.new_line();
+                write!(out, "private:");
+                out.new_line();
+                write!(out, "{}()", self.export_name);
+                out.open_brace();
+                out.close_brace(false);
+                out.new_line();
+                write!(out, "public:");
+                out.new_line();
+            }
+
             if config.language == Language::Cxx
                 && config
                     .enumeration
@@ -936,6 +983,29 @@ impl Source for Enum {
                 write!(out, "default: break;");
                 out.close_brace(false);
                 out.close_brace(false);
+
+                if config.language == Language::Cxx
+                    && config
+                        .enumeration
+                        .derive_tagged_enum_copy_assignment(&self.annotations)
+                {
+                    out.new_line();
+                    write!(
+                        out,
+                        "{}& operator=(const {}& {})",
+                        self.export_name, self.export_name, other
+                    );
+                    out.open_brace();
+                    write!(out, "if (this != &{})", other);
+                    out.open_brace();
+                    write!(out, "this->~{}();", self.export_name);
+                    out.new_line();
+                    write!(out, "new (this) {}({});", self.export_name, other);
+                    out.close_brace(false);
+                    out.new_line();
+                    write!(out, "return *this;");
+                    out.close_brace(false);
+                }
             }
 
             if let Some(body) = config.export.extra_body(&self.path) {
