@@ -7,17 +7,17 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::{Config, Language};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::ir::{
+use crate::bindgen::config::{Config, Language};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
     ToCondition, Type,
 };
-use bindgen::library::Library;
-use bindgen::mangle;
-use bindgen::monomorph::Monomorphs;
-use bindgen::writer::{Source, SourceWriter};
+use crate::bindgen::library::Library;
+use crate::bindgen::mangle;
+use crate::bindgen::monomorph::Monomorphs;
+use crate::bindgen::writer::{Source, SourceWriter};
 
 /// A type alias that is represented as a C typedef
 #[derive(Debug, Clone)]
@@ -77,20 +77,17 @@ impl Typedef {
             return;
         }
 
-        match self.aliased.get_root_path() {
-            Some(alias_path) => {
-                if out.contains_key(&alias_path) {
-                    warn!(
-                        "Multiple typedef's with annotations for {}. Ignoring annotations from {}.",
-                        alias_path, self.path
-                    );
-                    return;
-                }
-
-                out.insert(alias_path, self.annotations.clone());
-                self.annotations = AnnotationSet::new();
+        if let Some(alias_path) = self.aliased.get_root_path() {
+            if out.contains_key(&alias_path) {
+                warn!(
+                    "Multiple typedef's with annotations for {}. Ignoring annotations from {}.",
+                    alias_path, self.path
+                );
+                return;
             }
-            None => {}
+
+            out.insert(alias_path, self.annotations.clone());
+            self.annotations = AnnotationSet::new();
         }
     }
 
