@@ -5,15 +5,15 @@
 use std::collections::HashMap;
 use std::mem;
 
-use bindgen::bindings::Bindings;
-use bindgen::config::{Config, Language};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::error::Error;
-use bindgen::ir::{Constant, Enum, Function, Item, ItemContainer, ItemMap};
-use bindgen::ir::{OpaqueItem, Path, Static, Struct, Typedef, Union};
-use bindgen::monomorph::Monomorphs;
-use bindgen::ItemType;
+use crate::bindgen::bindings::Bindings;
+use crate::bindgen::config::{Config, Language};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::error::Error;
+use crate::bindgen::ir::{Constant, Enum, Function, Item, ItemContainer, ItemMap};
+use crate::bindgen::ir::{OpaqueItem, Path, Static, Struct, Typedef, Union};
+use crate::bindgen::monomorph::Monomorphs;
+use crate::bindgen::ItemType;
 
 #[derive(Debug, Clone)]
 pub struct Library {
@@ -41,15 +41,15 @@ impl Library {
         functions: Vec<Function>,
     ) -> Library {
         Library {
-            config: config,
-            constants: constants,
-            globals: globals,
-            enums: enums,
-            structs: structs,
-            unions: unions,
-            opaque_items: opaque_items,
-            typedefs: typedefs,
-            functions: functions,
+            config,
+            constants,
+            globals,
+            enums,
+            structs,
+            unions,
+            opaque_items,
+            typedefs,
+            functions,
         }
     }
 
@@ -74,6 +74,9 @@ impl Library {
         self.globals.for_all_items(|global| {
             global.add_dependencies(&self, &mut dependencies);
         });
+        self.constants.for_all_items(|constant| {
+            constant.add_dependencies(&self, &mut dependencies);
+        });
         for name in &self.config.export.include {
             let path = Path::new(name.clone());
             if let Some(items) = self.get_items(&path) {
@@ -112,6 +115,7 @@ impl Library {
 
         Ok(Bindings::new(
             self.config,
+            self.structs,
             constants,
             globals,
             items,
