@@ -9,17 +9,17 @@ use std::path::{Path as FilePath, PathBuf as FilePathBuf};
 
 use syn;
 
-use bindgen::bitflags;
-use bindgen::cargo::{Cargo, PackageRef};
-use bindgen::config::MacroExpansionConfig;
-use bindgen::error::Error;
-use bindgen::ir::{
+use crate::bindgen::bitflags;
+use crate::bindgen::cargo::{Cargo, PackageRef};
+use crate::bindgen::config::{Config, ParseConfig};
+use crate::bindgen::error::Error;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, Constant, Documentation, Enum, Function, GenericParams, ItemMap,
     OpaqueItem, Path, Static, Struct, Type, Typedef, Union,
 };
-use bindgen::utilities::{SynAbiHelpers, SynItemHelpers};
+use crate::bindgen::utilities::{SynAbiHelpers, SynItemHelpers};
 
-const STD_CRATES: &'static [&'static str] = &[
+const STD_CRATES: &[&str] = &[
     "std",
     "std_unicode",
     "alloc",
@@ -31,23 +31,18 @@ const STD_CRATES: &'static [&'static str] = &[
 type ParseResult = Result<Parse, Error>;
 
 /// Parses a single rust source file, not following `mod` or `extern crate`.
-pub fn parse_src(
-    src_file: &FilePath,
-    macro_expansion_config: &MacroExpansionConfig,
-) -> ParseResult {
+pub fn parse_src(src_file: &FilePath, config: &Config) -> ParseResult {
     let mod_name = src_file.file_stem().unwrap().to_str().unwrap();
+    let mut config = config.clone();
+    config.parse = ParseConfig {
+        parse_deps: true,
+        ..ParseConfig::default()
+    };
 
     let mut context = Parser {
         binding_crate_name: mod_name.to_owned(),
-        macro_expansion_config,
+        config: &config,
         lib: None,
-        parse_deps: true,
-        include: None,
-        exclude: Vec::new(),
-        expand: Vec::new(),
-        expand_all_features: true,
-        expand_default_features: true,
-        expand_features: None,
         parsed_crates: HashSet::new(),
         cache_src: HashMap::new(),
         cache_expanded_crate: HashMap::new(),
@@ -57,7 +52,7 @@ pub fn parse_src(
 
     let pkg_ref = PackageRef {
         name: mod_name.to_owned(),
-        version: "0.0.0".to_owned(),
+        version: None,
     };
 
     context.parse_mod(&pkg_ref, src_file)?;
@@ -69,28 +64,11 @@ pub fn parse_src(
 /// Inside a crate, `mod` and `extern crate` declarations are followed
 /// and parsed. To find an external crate, the parser uses the `cargo metadata`
 /// command to find the location of dependencies.
-pub(crate) fn parse_lib(
-    lib: Cargo,
-    macro_expansion_config: &MacroExpansionConfig,
-    parse_deps: bool,
-    include: &Option<Vec<String>>,
-    exclude: &[String],
-    expand: &[String],
-    expand_all_features: bool,
-    expand_default_features: bool,
-    expand_features: &Option<Vec<String>>,
-) -> ParseResult {
+pub(crate) fn parse_lib(lib: Cargo, config: &Config) -> ParseResult {
     let mut context = Parser {
         binding_crate_name: lib.binding_crate_name().to_owned(),
-        macro_expansion_config,
+        config,
         lib: Some(lib),
-        parse_deps: parse_deps,
-        include: include.clone(),
-        exclude: exclude.to_owned(),
-        expand: expand.to_owned(),
-        expand_all_features,
-        expand_default_features,
-        expand_features: expand_features.clone(),
         parsed_crates: HashSet::new(),
         cache_src: HashMap::new(),
         cache_expanded_crate: HashMap::new(),
@@ -106,16 +84,8 @@ pub(crate) fn parse_lib(
 #[derive(Debug, Clone)]
 struct Parser<'a> {
     binding_crate_name: String,
-    macro_expansion_config: &'a MacroExpansionConfig,
     lib: Option<Cargo>,
-    parse_deps: bool,
-
-    include: Option<Vec<String>>,
-    exclude: Vec<String>,
-    expand: Vec<String>,
-    expand_all_features: bool,
-    expand_default_features: bool,
-    expand_features: Option<Vec<String>>,
+    config: &'a Config,
 
     parsed_crates: HashSet<String>,
     cache_src: HashMap<FilePathBuf, Vec<syn::Item>>,
@@ -132,33 +102,51 @@ impl<'a> Parser<'a> {
             return false;
         }
 
-        if !self.parse_deps {
+        if !self.config.parse.parse_deps {
             return false;
         }
 
         // Skip any whitelist or blacklist for expand
-        if self.expand.contains(&pkg_name) {
+        if self.config.parse.expand.crates.contains(&pkg_name) {
             return true;
         }
 
         // If we have a whitelist, check it
-        if let Some(ref include) = self.include {
+        if let Some(ref include) = self.config.parse.include {
             if !include.contains(&pkg_name) {
                 return false;
             }
         }
 
         // Check the blacklist
-        return !STD_CRATES.contains(&pkg_name.as_ref()) && !self.exclude.contains(&pkg_name);
+        !STD_CRATES.contains(&pkg_name.as_ref()) && !self.config.parse.exclude.contains(&pkg_name)
     }
 
     fn parse_crate(&mut self, pkg: &PackageRef) -> Result<(), Error> {
         assert!(self.lib.is_some());
         self.parsed_crates.insert(pkg.name.clone());
 
-        // Before we do anything with this crate, we must first parse all of its dependencies.
-        // This is guaranteed to terminate because the crate-graph is acyclic (and even if it
-        // wasn't, we've already marked the crate as parsed in the line above).
+        // Check if we should use cargo expand for this crate
+        if self.config.parse.expand.crates.contains(&pkg.name) {
+            self.parse_expand_crate(pkg)?;
+        } else {
+            // Parse the crate before the dependencies otherwise the same-named idents we
+            // want to generate bindings for would be replaced by the ones provided
+            // by the first dependency containing it.
+            let crate_src = self.lib.as_ref().unwrap().find_crate_src(pkg);
+
+            match crate_src {
+                Some(crate_src) => self.parse_mod(pkg, crate_src.as_path())?,
+                None => {
+                    // This should be an error, but is common enough to just elicit a warning
+                    warn!(
+                        "Parsing crate `{}`: can't find lib.rs with `cargo metadata`.",
+                        pkg.name
+                    );
+                }
+            }
+        }
+
         for (dep_pkg, cfg) in self.lib.as_ref().unwrap().dependencies(&pkg) {
             if !self.should_parse_dependency(&dep_pkg.name) {
                 continue;
@@ -175,30 +163,19 @@ impl<'a> Parser<'a> {
             }
         }
 
-        // Check if we should use cargo expand for this crate
-        if self.expand.contains(&pkg.name) {
-            return self.parse_expand_crate(pkg);
-        }
-
-        // Otherwise do our normal parse
-        let crate_src = self.lib.as_ref().unwrap().find_crate_src(pkg);
-
-        match crate_src {
-            Some(crate_src) => self.parse_mod(pkg, crate_src.as_path()),
-            None => {
-                // This should be an error, but is common enough to just elicit a warning
-                warn!(
-                    "Parsing crate `{}`: can't find lib.rs with `cargo metadata`.",
-                    pkg.name
-                );
-                Ok(())
-            }
-        }
+        Ok(())
     }
 
     fn parse_expand_crate(&mut self, pkg: &PackageRef) -> Result<(), Error> {
         assert!(self.lib.is_some());
 
+        // If you want to expand the crate you run cbindgen on you might end up in an endless
+        // recursion if the cbindgen generation is triggered from build.rs. Hence don't run the
+        // expansion if the build was already triggered by cbindgen.
+        if std::env::var("_CBINDGEN_IS_RUNNING").is_ok() {
+            return Ok(());
+        }
+
         let mod_parsed = {
             if !self.cache_expanded_crate.contains_key(&pkg.name) {
                 let s = self
@@ -207,9 +184,9 @@ impl<'a> Parser<'a> {
                     .unwrap()
                     .expand_crate(
                         pkg,
-                        self.expand_all_features,
-                        self.expand_default_features,
-                        &self.expand_features,
+                        self.config.parse.expand.all_features,
+                        self.config.parse.expand.default_features,
+                        &self.config.parse.expand.features,
                     )
                     .map_err(|x| Error::CargoExpand(pkg.name.clone(), x))?;
                 let i = syn::parse_file(&s).map_err(|x| Error::ParseSyntaxError {
@@ -228,7 +205,7 @@ impl<'a> Parser<'a> {
 
     fn process_expanded_mod(&mut self, pkg: &PackageRef, items: &[syn::Item]) -> Result<(), Error> {
         self.out.load_syn_crate_mod(
-            &self.macro_expansion_config,
+            &self.config,
             &self.binding_crate_name,
             &pkg.name,
             Cfg::join(&self.cfg_stack).as_ref(),
@@ -239,24 +216,21 @@ impl<'a> Parser<'a> {
             if item.has_test_attr() {
                 continue;
             }
-            match *item {
-                syn::Item::Mod(ref item) => {
-                    let cfg = Cfg::load(&item.attrs);
-                    if let &Some(ref cfg) = &cfg {
-                        self.cfg_stack.push(cfg.clone());
-                    }
+            if let syn::Item::Mod(ref item) = *item {
+                let cfg = Cfg::load(&item.attrs);
+                if let Some(ref cfg) = cfg {
+                    self.cfg_stack.push(cfg.clone());
+                }
 
-                    if let Some((_, ref inline_items)) = item.content {
-                        self.process_expanded_mod(pkg, inline_items)?;
-                    } else {
-                        unreachable!();
-                    }
+                if let Some((_, ref inline_items)) = item.content {
+                    self.process_expanded_mod(pkg, inline_items)?;
+                } else {
+                    unreachable!();
+                }
 
-                    if cfg.is_some() {
-                        self.cfg_stack.pop();
-                    }
+                if cfg.is_some() {
+                    self.cfg_stack.pop();
                 }
-                _ => {}
             }
         }
 
@@ -303,7 +277,7 @@ impl<'a> Parser<'a> {
         items: &[syn::Item],
     ) -> Result<(), Error> {
         self.out.load_syn_crate_mod(
-            &self.macro_expansion_config,
+            &self.config,
             &self.binding_crate_name,
             &pkg.name,
             Cfg::join(&self.cfg_stack).as_ref(),
@@ -314,62 +288,57 @@ impl<'a> Parser<'a> {
             if item.has_test_attr() {
                 continue;
             }
-            match *item {
-                syn::Item::Mod(ref item) => {
-                    let next_mod_name = item.ident.to_string();
+            if let syn::Item::Mod(ref item) = *item {
+                let next_mod_name = item.ident.to_string();
 
-                    let cfg = Cfg::load(&item.attrs);
-                    if let &Some(ref cfg) = &cfg {
-                        self.cfg_stack.push(cfg.clone());
-                    }
+                let cfg = Cfg::load(&item.attrs);
+                if let Some(ref cfg) = cfg {
+                    self.cfg_stack.push(cfg.clone());
+                }
 
-                    if let Some((_, ref inline_items)) = item.content {
-                        self.process_mod(pkg, &mod_dir.join(&next_mod_name), inline_items)?;
+                if let Some((_, ref inline_items)) = item.content {
+                    self.process_mod(pkg, &mod_dir.join(&next_mod_name), inline_items)?;
+                } else {
+                    let next_mod_path1 = mod_dir.join(next_mod_name.clone() + ".rs");
+                    let next_mod_path2 = mod_dir.join(next_mod_name.clone()).join("mod.rs");
+
+                    if next_mod_path1.exists() {
+                        self.parse_mod(pkg, next_mod_path1.as_path())?;
+                    } else if next_mod_path2.exists() {
+                        self.parse_mod(pkg, next_mod_path2.as_path())?;
                     } else {
-                        let next_mod_path1 = mod_dir.join(next_mod_name.clone() + ".rs");
-                        let next_mod_path2 = mod_dir.join(next_mod_name.clone()).join("mod.rs");
-
-                        if next_mod_path1.exists() {
-                            self.parse_mod(pkg, next_mod_path1.as_path())?;
-                        } else if next_mod_path2.exists() {
-                            self.parse_mod(pkg, next_mod_path2.as_path())?;
-                        } else {
-                            // Last chance to find a module path
-                            let mut path_attr_found = false;
-                            for attr in &item.attrs {
-                                match attr.interpret_meta() {
-                                    Some(syn::Meta::NameValue(syn::MetaNameValue {
-                                        ident,
-                                        lit,
-                                        ..
-                                    })) => match lit {
-                                        syn::Lit::Str(ref path) if ident == "path" => {
-                                            path_attr_found = true;
-                                            self.parse_mod(pkg, &mod_dir.join(path.value()))?;
-                                            break;
-                                        }
-                                        _ => (),
-                                    },
+                        // Last chance to find a module path
+                        let mut path_attr_found = false;
+                        for attr in &item.attrs {
+                            match attr.parse_meta() {
+                                Ok(syn::Meta::NameValue(syn::MetaNameValue {
+                                    path, lit, ..
+                                })) => match lit {
+                                    syn::Lit::Str(ref path_lit) if path.is_ident("path") => {
+                                        path_attr_found = true;
+                                        self.parse_mod(pkg, &mod_dir.join(path_lit.value()))?;
+                                        break;
+                                    }
                                     _ => (),
-                                }
-                            }
-
-                            // This should be an error, but it's common enough to
-                            // just elicit a warning
-                            if !path_attr_found {
-                                warn!(
-                                    "Parsing crate `{}`: can't find mod {}`.",
-                                    pkg.name, next_mod_name
-                                );
+                                },
+                                _ => (),
                             }
                         }
-                    }
 
-                    if cfg.is_some() {
-                        self.cfg_stack.pop();
+                        // This should be an error, but it's common enough to
+                        // just elicit a warning
+                        if !path_attr_found {
+                            warn!(
+                                "Parsing crate `{}`: can't find mod {}`.",
+                                pkg.name, next_mod_name
+                            );
+                        }
                     }
                 }
-                _ => {}
+
+                if cfg.is_some() {
+                    self.cfg_stack.pop();
+                }
             }
         }
 
@@ -406,7 +375,7 @@ impl Parse {
     pub fn add_std_types(&mut self) {
         let mut add_opaque = |path: &str, generic_params: Vec<&str>| {
             let path = Path::new(path);
-            let generic_params: Vec<_> = generic_params.into_iter().map(|s| Path::new(s)).collect();
+            let generic_params: Vec<_> = generic_params.into_iter().map(Path::new).collect();
             self.opaque_items.try_insert(OpaqueItem::new(
                 path,
                 GenericParams(generic_params),
@@ -445,7 +414,7 @@ impl Parse {
 
     pub fn load_syn_crate_mod(
         &mut self,
-        macro_expansion_config: &MacroExpansionConfig,
+        config: &Config,
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
@@ -459,22 +428,28 @@ impl Parse {
             }
             match item {
                 syn::Item::ForeignMod(ref item) => {
-                    self.load_syn_foreign_mod(binding_crate_name, crate_name, mod_cfg, item);
+                    self.load_syn_foreign_mod(
+                        config,
+                        binding_crate_name,
+                        crate_name,
+                        mod_cfg,
+                        item,
+                    );
                 }
                 syn::Item::Fn(ref item) => {
-                    self.load_syn_fn(binding_crate_name, crate_name, mod_cfg, item);
+                    self.load_syn_fn(config, binding_crate_name, crate_name, mod_cfg, item);
                 }
                 syn::Item::Const(ref item) => {
-                    self.load_syn_const(binding_crate_name, crate_name, mod_cfg, item);
+                    self.load_syn_const(config, binding_crate_name, crate_name, mod_cfg, item);
                 }
                 syn::Item::Static(ref item) => {
-                    self.load_syn_static(binding_crate_name, crate_name, mod_cfg, item);
+                    self.load_syn_static(config, binding_crate_name, crate_name, mod_cfg, item);
                 }
                 syn::Item::Struct(ref item) => {
-                    self.load_syn_struct(crate_name, mod_cfg, item);
+                    self.load_syn_struct(config, crate_name, mod_cfg, item);
                 }
                 syn::Item::Union(ref item) => {
-                    self.load_syn_union(crate_name, mod_cfg, item);
+                    self.load_syn_union(config, crate_name, mod_cfg, item);
                 }
                 syn::Item::Enum(ref item) => {
                     self.load_syn_enum(crate_name, mod_cfg, item);
@@ -492,7 +467,7 @@ impl Parse {
                     }
                 }
                 syn::Item::Macro(ref item) => {
-                    self.load_builtin_macro(macro_expansion_config, crate_name, mod_cfg, item)
+                    self.load_builtin_macro(config, crate_name, mod_cfg, item)
                 }
                 _ => {}
             }
@@ -524,6 +499,7 @@ impl Parse {
     /// Enters a `extern "C" { }` declaration and loads function declarations.
     fn load_syn_foreign_mod(
         &mut self,
+        config: &Config,
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
@@ -535,31 +511,31 @@ impl Parse {
         }
 
         for foreign_item in &item.items {
-            match *foreign_item {
-                syn::ForeignItem::Fn(ref function) => {
-                    if crate_name != binding_crate_name {
-                        info!(
-                            "Skip {}::{} - (fn's outside of the binding crate are not used).",
-                            crate_name, &function.ident
-                        );
-                        return;
-                    }
-                    let path = Path::new(function.ident.to_string());
-                    match Function::load(path, &function.decl, true, &function.attrs, mod_cfg) {
-                        Ok(func) => {
-                            info!("Take {}::{}.", crate_name, &function.ident);
+            if let syn::ForeignItem::Fn(ref function) = *foreign_item {
+                if !config
+                    .parse
+                    .should_generate_top_level_item(crate_name, binding_crate_name)
+                {
+                    info!(
+                        "Skip {}::{} - (fn's outside of the binding crate are not used).",
+                        crate_name, &function.sig.ident
+                    );
+                    return;
+                }
+                let path = Path::new(function.sig.ident.to_string());
+                match Function::load(path, &function.sig, true, &function.attrs, mod_cfg) {
+                    Ok(func) => {
+                        info!("Take {}::{}.", crate_name, &function.sig.ident);
 
-                            self.functions.push(func);
-                        }
-                        Err(msg) => {
-                            error!(
-                                "Cannot use fn {}::{} ({}).",
-                                crate_name, &function.ident, msg
-                            );
-                        }
+                        self.functions.push(func);
+                    }
+                    Err(msg) => {
+                        error!(
+                            "Cannot use fn {}::{} ({}).",
+                            crate_name, &function.sig.ident, msg
+                        );
                     }
                 }
-                _ => {}
             }
         }
     }
@@ -567,30 +543,37 @@ impl Parse {
     /// Loads a `fn` declaration
     fn load_syn_fn(
         &mut self,
+        config: &Config,
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemFn,
     ) {
-        if crate_name != binding_crate_name {
+        if !config
+            .parse
+            .should_generate_top_level_item(crate_name, binding_crate_name)
+        {
             info!(
                 "Skip {}::{} - (fn's outside of the binding crate are not used).",
-                crate_name, &item.ident
+                crate_name, &item.sig.ident
             );
             return;
         }
 
         if let syn::Visibility::Public(_) = item.vis {
-            if item.is_no_mangle() && (item.abi.is_omitted() || item.abi.is_c()) {
-                let path = Path::new(item.ident.to_string());
-                match Function::load(path, &item.decl, false, &item.attrs, mod_cfg) {
+            if item.is_no_mangle() && (item.sig.abi.is_omitted() || item.sig.abi.is_c()) {
+                let path = Path::new(item.sig.ident.to_string());
+                match Function::load(path, &item.sig, false, &item.attrs, mod_cfg) {
                     Ok(func) => {
-                        info!("Take {}::{}.", crate_name, &item.ident);
+                        info!("Take {}::{}.", crate_name, &item.sig.ident);
 
                         self.functions.push(func);
                     }
                     Err(msg) => {
-                        error!("Cannot use fn {}::{} ({}).", crate_name, &item.ident, msg);
+                        error!(
+                            "Cannot use fn {}::{} ({}).",
+                            crate_name, &item.sig.ident, msg
+                        );
                     }
                 }
                 return;
@@ -600,18 +583,18 @@ impl Parse {
         // TODO
         if let syn::Visibility::Public(_) = item.vis {
         } else {
-            warn!("Skip {}::{} - (not `pub`).", crate_name, &item.ident);
+            warn!("Skip {}::{} - (not `pub`).", crate_name, &item.sig.ident);
         }
-        if (item.abi.is_omitted() || item.abi.is_c()) && !item.is_no_mangle() {
+        if (item.sig.abi.is_omitted() || item.sig.abi.is_c()) && !item.is_no_mangle() {
             warn!(
                 "Skip {}::{} - (`extern` but not `no_mangle`).",
-                crate_name, &item.ident
+                crate_name, &item.sig.ident
             );
         }
-        if item.abi.is_some() && !(item.abi.is_omitted() || item.abi.is_c()) {
+        if item.sig.abi.is_some() && !(item.sig.abi.is_omitted() || item.sig.abi.is_c()) {
             warn!(
                 "Skip {}::{} - (non `extern \"C\"`).",
-                crate_name, &item.ident
+                crate_name, &item.sig.ident
             );
         }
     }
@@ -681,12 +664,16 @@ impl Parse {
     /// Loads a `const` declaration
     fn load_syn_const(
         &mut self,
+        config: &Config,
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemConst,
     ) {
-        if crate_name != binding_crate_name {
+        if !config
+            .parse
+            .should_generate_top_level_item(crate_name, binding_crate_name)
+        {
             info!(
                 "Skip {}::{} - (const's outside of the binding crate are not used).",
                 crate_name, &item.ident
@@ -719,12 +706,16 @@ impl Parse {
     /// Loads a `static` declaration
     fn load_syn_static(
         &mut self,
+        config: &Config,
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemStatic,
     ) {
-        if crate_name != binding_crate_name {
+        if !config
+            .parse
+            .should_generate_top_level_item(crate_name, binding_crate_name)
+        {
             info!(
                 "Skip {}::{} - (static's outside of the binding crate are not used).",
                 crate_name, &item.ident
@@ -758,8 +749,14 @@ impl Parse {
     }
 
     /// Loads a `struct` declaration
-    fn load_syn_struct(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemStruct) {
-        match Struct::load(item, mod_cfg) {
+    fn load_syn_struct(
+        &mut self,
+        config: &Config,
+        crate_name: &str,
+        mod_cfg: Option<&Cfg>,
+        item: &syn::ItemStruct,
+    ) {
+        match Struct::load(&config.layout, item, mod_cfg) {
             Ok(st) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
                 self.structs.try_insert(st);
@@ -775,8 +772,14 @@ impl Parse {
     }
 
     /// Loads a `union` declaration
-    fn load_syn_union(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemUnion) {
-        match Union::load(item, mod_cfg) {
+    fn load_syn_union(
+        &mut self,
+        config: &Config,
+        crate_name: &str,
+        mod_cfg: Option<&Cfg>,
+        item: &syn::ItemUnion,
+    ) {
+        match Union::load(&config.layout, item, mod_cfg) {
             Ok(st) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
 
@@ -837,21 +840,21 @@ impl Parse {
 
     fn load_builtin_macro(
         &mut self,
-        macro_expansion_config: &MacroExpansionConfig,
+        config: &Config,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemMacro,
     ) {
         let name = match item.mac.path.segments.last() {
-            Some(ref n) => n.value().ident.to_string(),
+            Some(ref n) => n.ident.to_string(),
             None => return,
         };
 
-        if name != "bitflags" || !macro_expansion_config.bitflags {
+        if name != "bitflags" || !config.macro_expansion.bitflags {
             return;
         }
 
-        let bitflags = match bitflags::parse(item.mac.tts.clone()) {
+        let bitflags = match bitflags::parse(item.mac.tokens.clone()) {
             Ok(b) => b,
             Err(e) => {
                 warn!("Failed to parse bitflags invocation: {:?}", e);
@@ -860,7 +863,7 @@ impl Parse {
         };
 
         let (struct_, impl_) = bitflags.expand();
-        self.load_syn_struct(crate_name, mod_cfg, &struct_);
+        self.load_syn_struct(config, crate_name, mod_cfg, &struct_);
         // We know that the expansion will only reference `struct_`, so it's
         // fine to just do it here instead of deferring it like we do with the
         // other calls to this function.
